import React from 'react';
import PropTypes from 'prop-types';
import { Router } from 'dva/router';
import _ from 'lodash';
import config from './utils/config';
import { isAuthed, hasDomain } from './utils/auth';
import { fullPath, makePromise0 } from './utils/helper';
import App from './routes/app';
import { getMenus, getModuleInfo, getModuleLayout } from './data/modules';
import Monk from './routes/main/monk';

const { contextPath } = config;

const registerModel = (app, model) => {
// eslint-disable-next-line no-underscore-dangle
  if (!(app._models.filter(m => m.namespace === model.namespace).length === 1)) {
    app.model(model);
  }
};

const maybeLogin = (nextState, replace) => {
  if (!isAuthed()) {
    return replace(fullPath('/login'));
  }
};

const maybeSwitch = (nextState, replace) => {
  if (!hasDomain()) {
    return replace(fullPath('/domain'));
  }
};

const authenticated = (nextState, replace) => {
  maybeLogin(nextState, replace);
  maybeSwitch(nextState, replace);
};

const createRoute = async (app, { name, showName, modules, children }) => {
  // noinspection JSUnusedLocalSymbols
  return {
    path: name,
    name: showName,
    component: Monk,
    getChildRoutes(nextState, cb) {
      createRoutes(app, modules, children)
        .then((result) => {
          cb(null, result);
        })
        .catch((err) => {
          cb(err, null);
        });
    },
  };
};


const createRoutes = async (app, modules, groups) => {
  const routes = [];
  if (modules) {
    for (const module of modules) {
      let info;
      let layout;
      if (_.isString(module)) {
        info = await getModuleInfo(module);
        layout = await getModuleLayout(module);
      } else {
        info = module;
        layout = module.layout;
      }
      const { name, showName } = info;
      const route = {
        path: name,
        name: showName,
      };
      if (layout.route) {
        const modelBundle = await makePromise0(require('bundle!./models/main/modules/' + layout.route + '/index.js')); // eslint-disable-line import/no-dynamic-require, prefer-template, global-require
        registerModel(app, modelBundle);
        const routeBundle = await makePromise0(require('bundle!./routes/main/modules/' + layout.route + '/index.js')); // eslint-disable-line import/no-dynamic-require, prefer-template, global-require
        route.component = routeBundle;
        if (routeBundle.route) {
          for (const key in routeBundle.route) {
            if ({}.hasOwnProperty.call(routeBundle.route, key)) {
              route[key] = routeBundle.route[key];
            }
          }
        }
      } else {
        route.component = Monk;
      }
      routes.push(route);
    }
  }
  if (groups) {
    for (const group of groups) {
      routes.push(await createRoute(app, group));
    }
  }
  return routes;
};

function RouterConfig({ history, app }) {
  const routes = [
    {
      path: `${contextPath}/`,
      component: App,
      indexRoute: {
        onEnter: (nextState, replace) => replace(`${contextPath}/main`),
      },
      childRoutes: [
        {
          path: 'login',
          getComponent(nextState, cb) {
            require.ensure([], (require) => {
              registerModel(app, require('./models/login'));
              cb(null, require('./routes/login'));
            }, 'login');
          },
        },
        {
          path: 'domain',
          onEnter: maybeLogin,
          getComponent(nextState, cb) {
            require.ensure([], (require) => {
              registerModel(app, require('./models/domain'));
              cb(null, require('./routes/domain'));
            }, 'domain');
          },
        },
        {
          path: 'main',
          onEnter: authenticated,
          getComponent(nextState, cb) {
            require.ensure([], (require) => {
              registerModel(app, require('./models/main'));
              cb(null, require('./routes/main'));
            }, 'main');
          },
          getChildRoutes: (nextState, cb) => {
            getMenus()
              .then((menus) => {
                createRoutes(app, [], menus)
                  .then((result) => {
                    cb(null, result);
                  })
                  .catch((err) => {
                    cb(err, null);
                  });
              })
              .catch((err) => {
                cb(err, null);
              });
            // fetchModules()
            //   .then(() => {
            //     const lvl0 = [];
            //     foreachModule((module) => {
            //       if (!module.parent) {
            //         lvl0.push(module);
            //       }
            //     });
            //     createModuleRoutes(app, lvl0)
            //       .then((result) => {
            //         cb(null, result);
            //       })
            //       .catch((err) => {
            //         cb(err, null);
            //       });
            //   })
            //   .catch((err) => {
            //     cb(err, null);
            //   });
          },
        },
      ],
    },
  ];
  return (
    <Router history={history} routes={routes} />
  );
}

RouterConfig.propTypes = {
  history: PropTypes.object,
  app: PropTypes.object,
};

export default RouterConfig;
